/****************************************************************************
 *  Familia Lignum - Genealogical program                                   *
 *  Copyright (C) 2011-2012 Rafał Długołęcki <rafal@dlugolecki.net.pl>      *
 *                                                                          *
 *  This program is free software; you can redistribute it and/or modify    *
 *  it under the terms of the GNU General Public License as published by    *
 *  the Free Software Foundation; version 2 of the License.                 *
 *                                                                          *
 *  This program is distributed in the hope that it will be useful,         *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License along *
 *  with this program; if not, write to the Free Software Foundation, Inc., *
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             *
 *                                                                          *
 ****************************************************************************/

#include <iostream>
#include <istream>
#include <sstream>
#include <sigc++/sigc++.h>
#include <glibmm/convert.h> //For Glib::ConvertError
#include <zipios++/fcollexceptions.h>
#include <zipios++/zipinputstream.h>
#include <zipios++/simplesmartptr.h>
#include <libxml++/libxml++.h>
#include <libxml/tree.h>

#include "gno.h"

using namespace std;
using namespace zipios;

namespace file {

Gno::Gno(const _String filename)
  : file::File(filename), xmlpp::SaxParser() {
  set_substitute_entities(true);

  cout << "Gno/open" << endl;
  try {
    ZipInputStream is(filename);
    parse_stream(is);
  }
  catch(IOException &e) {
    cerr << "IOException caught in main:" << endl;
    cerr << e.what() << endl;
  }
  catch(Exception &e) {
    cerr << "Some nasty error occured " << e.what() << endl;
  }
}

Gno::~Gno() {
}

void Gno::on_start_element(const _String& name,
                            const AttributeList& attributes) {
  if(name == "Individual") {
    _Individual ind;
    ind.setId(parseIndividualIdentifier(attributes[0].value));
    current_individual = ind;
    currentCollection = individual;
  }
  else if(name == "Family") {
    _Family fam(parseFamilyIdentifier(attributes[0].value));
    current_family = fam;
    currentCollection = family;
  }
  else if(name == "GenoMap") {
    _View view;
    for(unsigned int i = 0; i < attributes.size(); i++) {
      const xmlpp::SaxParser::Attribute attr = attributes[i];
      view.parse(attr.name, attr.value); 
    }
    views.push_back(view);
  }

  switch(currentCollection) {
  case individual:
    if(name == "Name")         { individualState = individualName; }
    else if(name == "Display") { individualState = nameDisplay; }
    else if(name == "First")   { individualState = nameFirst; }
    else if(name == "Middle")  { individualState = nameMiddle; }
    else if(name == "Last")    { individualState = nameLast; }

    else if(name == "Pictures") { individualState = pictures; }
    else if(name == "Contacts") { individualState = contacts; }
    else if(name == "Position") { individualState = position;
      //If there are loaded genomaps, and no genomap is specified for
      //this individual. Add him to the first genomap.
      if(!views.empty()) {
	current_individual.viewId() = views[0].name();
      }
      
      for(int i = 0; i < attributes.size(); i++) {
	if(attributes[i].name == "BoundaryRect") {
	  current_individual.boundary().parse(attributes[i].value, ',');
	}
	else if(attributes[i].name == "GenoMap") {
	  current_individual.viewId() = attributes[i].value;
	}
      }
    }
    else if(name == "Gender") { individualState = gender; }
    else if(name == "IsDead") { individualState = isDead; }
    else if(name == "Birth") { individualState = birth; }
      else if(name == "Date") { individualState = birthDate; }
      else if(name == "Place") { individualState = birthPlace; }
        else if(name == "Baptism") { individualState = birthBaptism; }
        else if(name == "Date"
		&& individualState == birthBaptism) { individualState = birthBaptismDate; }
        else if(name == "Place") { individualState = birthBaptismPlace; }
        else if(name == "Officiator") { individualState = birthBaptismOfficiator; }
        else if(name == "Godfather") { individualState = birthBaptismGodfather; }
        else if(name == "Godmother") { individualState = birthBaptismGodmother; }
    else if(name == "Comment") { individualState = comment; }
    else if(name == "Educations") { individualState = educations; }
    else if(name == "Hyperlink") { individualState = hyperlink; }
    else if(name == "child_no") { individualState = child_no; }
    break;
  case family:
    break;
  }
}

void Gno::on_end_element(const Glib::ustring& n) {
  _String name = Glib::locale_to_utf8(n);
  if(name == "Individual") {
    individuals[current_individual.getId()] = current_individual;
    cout << "Saving individual: " << current_individual.getId() << " "  << current_individual.position().get_x() << endl;
  }
  else if(name == "Individual") {
    families[current_family.getId()] = current_family;
  }
  if(currentCollection == individual) {
    individualState = unknown;
  }
}

void Gno::on_characters(const Glib::ustring& chars) {
  switch(currentCollection) {
  case individual:
    switch(individualState) {
    case individualName:
    case nameDisplay:
      break;
    case nameFirst: {
      current_individual.name().first += chars;
      break;
    }
    case nameMiddle: {
      current_individual.name().middle += chars;
      break;
    }
    case nameLast: {
      current_individual.name().last += chars;
      break;
    }
    case pictures:
    case contacts: {
      break;
    }
    case position: {
      current_individual.position().parse(chars);
      break;
    }
    case gender: {
      current_individual.setGender(parseIndividualGender(chars));
      break;
    }
    case isDead:current_individual.setDead(parseIndividualDeadState(chars));
      break;
//    case birth:
    case birthDate:
    case birthPlace:
    case birthBaptism:
    case birthBaptismDate:
    case birthBaptismPlace:
//    case birthBaptismOfficiator:
    case birthBaptismGodfather:
    case birthBaptismGodmother:
    case comment:
    case educations:
//    case hyperlink:
    case child_no:
      break;
    }
  case family:
    switch(familyState) {
//    case position:
//    case positionTop:
    case positionTopLeft:
    case positionTopRight:
//    case positionBottom:
    case positionBottomLeft:
    case positionBottomRight:
    case familyLine:
    case relation:
//    case children:
    case unions:
      break;
    }
    break;
  }
}

int parseFamilyIdentifier(_String identifier) {
  int id;
  stringstream(identifier.substr(3)) >> id;
  return id;
}

int parseIndividualIdentifier(_String identifier) {
  int id;
  stringstream(identifier.substr(3)) >> id;
  return id;
}

gender_type parseIndividualGender(_String str) {
  if(str == "M")      {
    return male;
  }
  else if(str == "F") {
    return female;
  }
  return unknownGender;
}

bool parseIndividualDeadState(_String str) {
  if(str == "Y") {
    return true;
  }
  return false;
}

}
